﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
    
//-------------------------------------------------------------
// Часть 0 расположена в отдельном файле AlarmClock.Designer.cs
//-------------------------------------------------------------
    
// Часть 1
namespace MyCompany.MyComponents
{
    public partial class AlarmClock : Component
    {
        public AlarmClock()
        {
            InitializeComponent();
        }
    
        public AlarmClock(IContainer container)
        {
            container.Add(this);
    
            InitializeComponent();
        }
    }
}
    
// Часть 2
namespace MyCompany.MyComponents
{
    // Часть класса с определением полей и свойств
    partial class AlarmClock
    {
        // Объявляем внутренние поля
        bool alarmFired = false; // Состояние будильника (запущен/незапущен)
        DateTime alarmTime = DateTime.Now; // Хранит время запуска будильника
    
        // Вложенный класс аргументов, доступных в обработчике
        // Расширяет библиотечный класс аргументов событий
        public class AlarmEventArgs : EventArgs
        {
            // Поле
            DateTime time;
    
            // Свойство
            public DateTime Time
            {
                get { return time; }
                set { time = value; }
            }
        }
    
        // Свойство возвращает текущее время компьютера
        public DateTime CurrentTime
        {
            get { return DateTime.Now; }
        }
    
        // Свойство для чтения и установки времени запуска будильника
        public DateTime AlarmTime
        {
            get { return alarmTime; }
            set
            {
                if (value != alarmTime)
                {
                    alarmTime = value;
                    alarmFired = false;
                }
            }
        }
    
        // Свойство для проверки состояния хода системного таймера, 
        // его остановки и запуска
        public bool Enabled
        {
            get { return timer1.Enabled; }
            set
            {
                timer1.Enabled = value;
                if (value)
                    alarmFired = false;
            }
        }
    }
}
 
// Часть 3
namespace MyCompany.MyComponents
{
    // Часть класса с определением собственного события
    // стандартным способом без использования базового поля
    partial class AlarmClock
    {
        // Объявляем делегат для события. В аргументах делегата
        // нужно указать полное имя вложенного класса
        public delegate void AlarmHandler(object sender, 
            AlarmClock.AlarmEventArgs e);
    
        // Объявляем событие с доступностью не ниже делегата
        public event AlarmHandler Alarm;
    
        // Метод диспетчеризации события. Виртуальный и защищенный
        // для возможности переопределения в будущих потомках
        protected virtual void OnAlarm(AlarmEventArgs args)
        {
            // Проверяем наличие зарегистрированных 
            // обработчиков и генерируем событие
            if (Alarm != null)
                Alarm(this, args);
        }
    }
}
    
// Часть 4
namespace MyCompany.MyComponents
{
    // Часть класса с определением обработчика события Tick 
    // системного таймера System.Windows.Forms.Timer timer1
    partial class AlarmClock
    {
        // Обработчик события срабатывания системного таймера, созданного,
        // настроенного и зарегистрированного через визуальный конструктор.
        // Обработчик выполняется каждую секунду и проверяет, пришла ли пора
        // генерировать наше событие, на которое подпишется клиент компонента
        private void TimerHandler(object sender, EventArgs e)
        {
            DateTime now; // Текущее системное время
            AlarmEventArgs args;
    
            // В режиме разработки не выполнять
            if (!this.DesignMode)
            {
                now = DateTime.Now;
                // Если будильник не запущен и пришла пора запускать
                if (!alarmFired && now >= alarmTime)
                {
                    // Создаем объект для передачи аргументов в событии
                    args = new AlarmEventArgs();
                    // Заполняем объект текущим временем
                    args.Time = now;
                    // Вызываем метод диспетчеризации события Alarm
                    this.OnAlarm(args);

                    // Поднимаем флаг "Будильник запущен"
                    alarmFired = true;
                }
            }
        }
    }
}

// Часть 5
namespace MyCompany.MyComponents
{
    // Часть класса с определением обработчика события Tick 
    // системного таймера System.Windows.Forms.Timer timer2
    partial class AlarmClock
    {
        // Объявляем внутренние поля
        bool alarmFiredExt = false; // Состояние будильника (запущен/незапущен)
        DateTime alarmTimeExt = DateTime.Now; // Хранит время запуска будильника
    
        // Свойство для чтения и установки времени запуска будильника
        public DateTime AlarmTimeExt
        {
            get { return alarmTimeExt; }
            set
            {
                if (value != alarmTimeExt)
                {
                    alarmTimeExt = value;
                    alarmFiredExt = false;
                }
            }
        }
    
        // Свойство для проверки состояния хода второго
        // системного таймера, его остановки и запуска
        public bool EnabledExt
        {
            get { return timer2.Enabled; }
            set
            {
                timer2.Enabled = value;
                if (value)
                    alarmFiredExt = false;
            }
        }
        
        // Обработчик события срабатывания второго системного таймера
        private void TimerHandlerExt(object sender, EventArgs e)
        {
            DateTime now; // Текущее системное время
            AlarmEventArgs args;
    
            // В режиме разработки не выполнять
            if (!this.DesignMode)
            {
                now = DateTime.Now;
                // Если будильник не запущен и пришла пора запускать
                if (!alarmFiredExt && now >= alarmTimeExt)
                {
                    // Создаем объект для передачи аргументов в событии
                    args = new AlarmEventArgs();
                    // Заполняем объект текущим временем
                    args.Time = now;
                    // Вызываем метод диспетчеризации события AlarmExt
                    this.OnAlarmExt(args);
    
                    // Поднимаем флаг "Будильник запущен"
                    alarmFiredExt = true;
                }
            }
        }
    }
}
    
// Часть 6 Extension
namespace MyCompany.MyComponents
{
    // Часть класса с определением собственного события
    // стандартным способом без использования базового поля
    partial class AlarmClock
    {
        // Закрытое поле-ссылка на экземпляр делегата
        // Для объявления поля решили использовать уже существующий 
        // делегат, но можно объявить и другой делегат с той же сигнатурой
        private AlarmHandler m_AlarmExt;
    
        // Объявляем пользовательское событие
        // с возможностью контроля доступа к нему
        public event AlarmHandler AlarmExt
        {
            // Эта функция будет вызвана при  попытке 
            // добавления обработчика в список вызова события
            add
            {
                // Здесь что-то можно проконтролировать, например, что
                // произошла попытка добавить обработчик в список вызова события
                Console.WriteLine("Добавлен обработчик в список события AlarmExt");
    
                // Расширяем список объекта-делегата, ссылающегося на обработчики события
                m_AlarmExt += value;
            }
            // Эта функция будет вызвана при попытке 
            // изъятия обработчика из списка вызова события
            remove
            {
                // Здесь что-то можно проконтролировать, например, что
                // произошла попытка изъять обработчик из списка вызова события
                Console.WriteLine("Изъят обработчик из списка события AlarmExt");
    
                // Удалим обработчик из списка объекта-делегата
                m_AlarmExt -= value;
            }
        }
    
        // Метод диспетчеризации события. Виртуальный и защищенный
        // для возможности переопределения в будущих потомках
        protected virtual void OnAlarmExt(AlarmEventArgs args)
        {
            // Проверяем наличие зарегистрированных обработчиков 
            // и генерируем событие через поле-делегат
            if (m_AlarmExt != null)
                m_AlarmExt(this, args);
        }
    }
}
