﻿using System;
using System.ComponentModel;
    
namespace MyCompany.MyComponents
{
    // Контролирующий класс, предоставляющий возможность 
    // развертывания и редактирования свойств вложенных объектов
    public class SomeObjectConverter : ExpandableObjectConverter
    {
        // Проверяем возможность преобразования типов
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            // Проверяем тип составной строки 
            if (sourceType == typeof(System.String))
                return true;
            else
                return base.CanConvertFrom(context, sourceType);
        }
    
        // Преобразуем составную строку в значения свойств
        public override object ConvertFrom(ITypeDescriptorContext context, 
            System.Globalization.CultureInfo culture, object value)
        {
            string sourceStr = value as System.String;
            // Если очистить строку, восстанавливаем исходный объект
            if (sourceStr == "")
            {
                return new SomeObject();
            }
            else
            {
                // Расщепляем строку свойств
                char[] separator ={ ';' };
                string[] PropStrings = sourceStr.Split(separator);
                // Проверяем число элементов строки и в случае несовпадения
                // генерируем исключение, перехватываемое оболочкой 
                if (PropStrings.Length != 2)
                    throw new ArgumentException(
                        "Требуется формат: два значения с разделителем ';'");
                // Создаем новый объект и присваиваем свойствам новые значения
                SomeObject obj = new SomeObject();
                obj.Prop1 = Convert.ToInt32(PropStrings[0].Trim());
                obj.Prop2 = PropStrings[1].Trim();
                return obj;
            }
        }
    
        // Преобразуем значения свойств в составную строку
        public override object ConvertTo(ITypeDescriptorContext context, 
            System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            SomeObject obj = (SomeObject)value;
            return obj.Prop1.ToString() + "; " + obj.Prop2;
        }
    }
}